@@ -7,10 +7,9 @@ use warnings;
 use 5.008;
 
 use Carp       ();
-use Hash::Util ();
 use UNIVERSAL::Object;
 
-our $VERSION   = '0.08';
+our $VERSION   = '0.09';
 our $AUTHORITY = 'cpan:STEVAN';
 
 our @ISA; BEGIN { @ISA = ('UNIVERSAL::Object') }
@@ -18,10 +17,20 @@ our @ISA; BEGIN { @ISA = ('UNIVERSAL::Object') }
 sub BLESS {
     my $self = $_[0]->SUPER::BLESS( $_[1] );
 
-    Carp::croak('Immutable objects must use a HASH ref REPR type, not '.$self)
-        unless $self =~ /\=HASH\(0x/;
+    if ( $self =~ /\=HASH\(0x/ ) {
+        require Hash::Util;
+        Hash::Util::lock_hash( $self );
+    }
+    elsif ( $self =~ /\=ARRAY\(0x/ ) {
+        Internals::SvREADONLY( @$self, 1 );
+    }
+    elsif ( $self =~ /\=SCALAR\(0x/ or $self =~ /\=REF\(0x/ ) {
+        Internals::SvREADONLY( $$self, 1 );
+    }
+    else {
+        Carp::croak('Unsupported REPR type: '.$self);
+    }
 
-    Hash::Util::lock_hash( %$self );
     return $self;
 }
 
@@ -37,7 +46,7 @@ UNIVERSAL::Object::Immutable - Another useful base class
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
@@ -50,6 +59,13 @@ You can use this class in the same manner that you would use
 L<UNIVERSAL::Object>, the only difference is that the instances
 created will be immutable.
 
+=head2 Supported REPR types
+
+This module will attempt to do the right type of locking for
+the three main instance types; C<SCALAR>, C<REF>, C<ARRAY> and
+C<HASH>, all other instance types are unsupported and will
+throw an error.
+
 =head2 Why Immutability?
 
 Immutable data structures are unable to be changed after they are
@@ -140,10 +156,11 @@ the same effect as the multiple inheritance.
 =head2 Compatibility Note
 
 This class requires Perl 5.8.0 or later, this is because it
-depends on the L<Hash::Util> module, which was first introduced
-in that version of Perl. Since this is an optional component,
-we have not bumped the version requirement for the entire
-distribution, only for this module specifically.
+depends on the L<Hash::Util> module and the C<Internals::SvREADONLY>
+feature, both of which were first introduced in that version of Perl.
+Since this is an optional component, we have not bumped the version
+requirement for the entire distribution, only for this module
+specifically.
 
 =head1 AUTHOR
 
